@@ -1,14 +1,17 @@
 from collections import defaultdict
 from itertools import chain
 from typing import Dict, Iterable, List, Optional, Tuple
+from math import ceil
+from datetime import datetime
 
 from maggma.core import Builder, Store
 from maggma.utils import grouper
+from pymatgen.analysis.structure_matcher import ElementComparator, StructureMatcher
 from pymatgen.core.structure import Structure
 
 from emmet.builders.settings import EmmetBuildSettings
 from emmet.core.provenance import ProvenanceDoc, SNLDict
-from emmet.core.utils import group_structures
+from emmet.core.utils import group_structures, get_sg
 
 
 class ProvenanceBuilder(Builder):
@@ -62,7 +65,7 @@ class ProvenanceBuilder(Builder):
         # Find all formulas for materials that have been updated since this
         # builder was last ran
         q = self.query
-        updated_materials = self.provenance.newer_in(self.materials, criteria=q, exhaustive=True,)
+        updated_materials = self.provenance.newer_in(self.materials, criteria=q, exhaustive=True)
         forms_to_update = set(self.materials.distinct("formula_pretty", {"material_id": {"$in": updated_materials}}))
 
         # Find all new SNL formulas since the builder was last run
@@ -74,13 +77,16 @@ class ProvenanceBuilder(Builder):
         forms_avail = set(self.materials.distinct("formula_pretty", self.query))
         forms_to_update = forms_to_update & forms_avail
 
-        self.logger.info(
-            f"Found {len(forms_to_update)} new/updated systems to distribute to workers "
-            f"in chunks of {len(forms_to_update)/number_splits}"
+        mat_ids = set(self.materials.distinct("material_id", {"formula_pretty": {"$in": list(forms_to_update)}})) & set(
+            updated_materials
         )
 
-        for chunk in grouper(forms_to_update, number_splits):
-            yield {"formula_pretty": {"$in": chunk}}
+        N = ceil(len(mat_ids) / number_splits)
+
+        self.logger.info(f"Found {len(mat_ids)} new/updated systems to distribute to workers " f"in {N} chunks.")
+
+        for chunk in grouper(mat_ids, N):
+            yield {"query": {"material_id": {"$in": chunk}}}
 
     def get_items(self) -> Tuple[List[Dict], List[Dict]]:  # type: ignore
         """
@@ -96,7 +102,7 @@ class ProvenanceBuilder(Builder):
         # Find all formulas for materials that have been updated since this
         # builder was last ran
         q = self.query
-        updated_materials = self.provenance.newer_in(self.materials, criteria=q, exhaustive=True,)
+        updated_materials = self.provenance.newer_in(self.materials, criteria=q, exhaustive=True)
         forms_to_update = set(self.materials.distinct("formula_pretty", {"material_id": {"$in": updated_materials}}))
 
         # Find all new SNL formulas since the builder was last run
@@ -108,45 +114,47 @@ class ProvenanceBuilder(Builder):
         forms_avail = set(self.materials.distinct("formula_pretty", self.query))
         forms_to_update = forms_to_update & forms_avail
 
-        self.logger.info(f"Found {len(forms_to_update)} new/updated systems to process")
+        mat_ids = set(self.materials.distinct("material_id", {"formula_pretty": {"$in": list(forms_to_update)}})) & set(
+            updated_materials
+        )
 
-        self.total = len(forms_to_update)
+        self.total = len(mat_ids)
 
-        for formulas in grouper(forms_to_update, self.chunk_size):
-            snls = []  # type: list
-            for source in self.source_snls:
-                snls.extend(source.query(criteria={"formula_pretty": {"$in": formulas}}))
+        self.logger.info(f"Found {self.total} new/updated systems to process")
 
-            mats = list(
-                self.materials.query(
-                    properties=[
-                        "material_id",
-                        "last_updated",
-                        "structure",
-                        "initial_structures",
-                        "formula_pretty",
-                        "deprecated",
-                    ],
-                    criteria={"formula_pretty": {"$in": formulas}},
-                )
+        for mat_id in mat_ids:
+
+            mat = self.materials.query_one(
+                properties=[
+                    "material_id",
+                    "last_updated",
+                    "structure",
+                    "initial_structures",
+                    "formula_pretty",
+                    "deprecated",
+                ],
+                criteria={"material_id": mat_id},
             )
 
-            form_groups = defaultdict(list)
+            snls = []  # type: list
+            for source in self.source_snls:
+                snls.extend(source.query(criteria={"formula_pretty": mat["formula_pretty"]}))
+
+            snl_groups = defaultdict(list)
             for snl in snls:
-                form_groups[snl["formula_pretty"]].append(snl)
+                struc = Structure.from_dict(snl)
+                snl_sg = get_sg(struc)
+                struc.snl = SNLDict(**snl)
+                snl_groups[snl_sg].append(struc)
 
-            mat_groups = defaultdict(list)
-            for mat in mats:
-                mat_groups[mat["formula_pretty"]].append(mat)
+            mat_sg = get_sg(Structure.from_dict(mat["structure"]))
 
-            for formula, snl_group in form_groups.items():
+            snl_structs = snl_groups[mat_sg]
 
-                mat_group = mat_groups[formula]
+            self.logger.debug(f"Found {len(snl_structs)} potential snls for {mat_id}")
+            yield mat, snl_structs
 
-                self.logger.debug(f"Found {len(snl_group)} snls and {len(mat_group)} mats")
-                yield mat_group, snl_group
-
-    def process_item(self, item) -> List[Dict]:
+    def process_item(self, item) -> Dict:
         """
         Matches SNLS and Materials
         Args:
@@ -154,36 +162,43 @@ class ProvenanceBuilder(Builder):
         Returns:
             list(dict): a list of collected snls with material ids
         """
-        mats, source_snls = item
-        formula_pretty = mats[0]["formula_pretty"]
-        snl_docs = list()
+        mat, snl_structs = item
+        formula_pretty = mat["formula_pretty"]
+        snl_doc = None
         self.logger.debug(f"Finding Provenance {formula_pretty}")
 
         # Match up SNLS with materials
-        for mat in mats:
-            matched_snls = list(self.match(source_snls, mat))
 
-            if len(matched_snls) > 0:
-                doc = ProvenanceDoc.from_SNLs(
-                    material_id=mat["material_id"],
-                    structure=Structure.from_dict(mat["structure"]),
-                    snls=matched_snls,
-                    deprecated=mat["deprecated"],
-                )
+        matched_snls = self.match(snl_structs, mat)
 
-                doc.authors.append(self.settings.DEFAULT_AUTHOR)
-                doc.history.append(self.settings.DEFAULT_HISTORY)
-                doc.references.append(self.settings.DEFAULT_REFERENCE)
+        if len(matched_snls) > 0:
+            doc = ProvenanceDoc.from_SNLs(
+                material_id=mat["material_id"],
+                structure=Structure.from_dict(mat["structure"]),
+                snls=matched_snls,
+                deprecated=mat["deprecated"],
+            )
+        else:
+            doc = ProvenanceDoc(
+                material_id=mat["material_id"],
+                structure=Structure.from_dict(mat["structure"]),
+                deprecated=mat["deprecated"],
+                created_at=datetime.utcnow(),
+            )
 
-                snl_docs.append(doc.dict(exclude_none=True))
+        doc.authors.append(self.settings.DEFAULT_AUTHOR)
+        doc.history.append(self.settings.DEFAULT_HISTORY)
+        doc.references.append(self.settings.DEFAULT_REFERENCE)
 
-        return snl_docs
+        snl_doc = doc.dict(exclude_none=True)
 
-    def match(self, snls, mat):
+        return snl_doc
+
+    def match(self, snl_structs, mat):
         """
         Finds a material doc that matches with the given snl
         Args:
-            snl ([dict]): the snls list
+            snl_structs ([dict]): the snls struct list
             mat (dict): a materials doc
         Returns:
             generator of materials doc keys
@@ -192,22 +207,25 @@ class ProvenanceBuilder(Builder):
         m_strucs = [Structure.from_dict(mat["structure"])] + [
             Structure.from_dict(init_struc) for init_struc in mat["initial_structures"]
         ]
-        snl_strucs = []
-        for snl in snls:
-            struc = Structure.from_dict(snl)
-            struc.snl = SNLDict(**snl)
-            snl_strucs.append(struc)
 
-        groups = group_structures(
-            m_strucs + snl_strucs,
+        sm = StructureMatcher(
             ltol=self.settings.LTOL,
             stol=self.settings.STOL,
             angle_tol=self.settings.ANGLE_TOL,
-            # comparator=OrderDisorderElementComparator(),
+            primitive_cell=True,
+            scale=True,
+            attempt_supercell=False,
+            allow_subset=False,
+            comparator=ElementComparator(),
         )
 
-        matched_groups = [group for group in groups if any(not hasattr(struc, "snl") for struc in group)]
-        snls = [struc.snl for group in matched_groups for struc in group if hasattr(struc, "snl")]
+        snls = []
+
+        for s in m_strucs:
+            for snl_struc in snl_structs:
+                if sm.fit(s, snl_struc):
+                    if snl_struc.snl not in snls:
+                        snls.append(snl_struc.snl)
 
         self.logger.debug(f"Found {len(snls)} SNLs for {mat['material_id']}")
         return snls
@@ -216,8 +234,7 @@ class ProvenanceBuilder(Builder):
         """
         Inserts the new SNL docs into the SNL collection
         """
-
-        snls = list(filter(None, chain.from_iterable(items)))
+        snls = list(filter(None, items))
 
         if len(snls) > 0:
             self.logger.info(f"Found {len(snls)} SNLs to update")
