@@ -2,6 +2,7 @@ import warnings
 from collections import defaultdict
 from itertools import chain
 from typing import Dict, Iterable, Iterator, List, Optional, Set
+from math import ceil
 
 from maggma.core import Builder, Store
 from maggma.utils import grouper
@@ -82,8 +83,11 @@ class ThermoBuilder(Builder):
             if chemsys not in to_process_chemsys:
                 to_process_chemsys |= chemsys_permutations(chemsys)
 
-        for chemsys_chunk in grouper(to_process_chemsys, number_splits):
-            yield {"chemsys": {"$in": list(chemsys_chunk)}}
+        N = ceil(len(to_process_chemsys) / number_splits)
+
+        for chemsys_chunk in grouper(to_process_chemsys, N):
+
+            yield {"query": {"chemsys": {"$in": list(chemsys_chunk)}}}
 
     def get_items(self) -> Iterator[List[Dict]]:
         """
@@ -264,9 +268,7 @@ class ThermoBuilder(Builder):
 
         return all_entries
 
-    def get_updated_chemsys(
-        self,
-    ) -> Set:
+    def get_updated_chemsys(self,) -> Set:
         """Gets updated chemical system as defined by the updating of an existing material"""
 
         updated_mats = self.thermo.newer_in(self.materials, criteria=self.query)
